# GOAL: generate a VPC that correctly takes into account the dropouts
library("mlxR")


#===============================================================================
# VPC as in Monolix: the same PD observation times as in the data set are used
# whatever the new simulated dropout (death) time is
#===============================================================================

project.name <- "PDTTE_dropout"

# read empirical percentiles from Monolix output files
bins <- read.csv(paste0(project.name,"/ChartsData/VisualPredictiveCheck/y1_bins.txt"), header=T)$bins_values
binsmiddle <- c(bins[1], (bins[2:length(bins)]+bins[1:(length(bins)-1)])/2, bins[length(bins)])
empirical <- read.csv(paste0(project.name,"/ChartsData/VisualPredictiveCheck/y1_percentiles.txt"), header=T)
empirical$bins_middles <- binsmiddle[2:(length(binsmiddle)-1)]

# simulate data sets (using replicates)
res1a <- simulx(project=paste0(project.name,".mlxtran"),
              nrep = 500)

# assign simulated data to bins
res1a$y1$binindex <- findInterval(res1a$y1$time, bins, all.inside = T)

# calculate percentiles for each replicate (data set) and each bin
percentile10 <- aggregate(res1a$y1$y1, by=list(bin=res1a$y1$binindex,rep=res1a$y1$rep), FUN='quantile', probs=c(0.1))
percentile50 <- aggregate(res1a$y1$y1, by=list(bin=res1a$y1$binindex,rep=res1a$y1$rep), FUN='quantile', probs=c(0.5))
percentile90 <- aggregate(res1a$y1$y1, by=list(bin=res1a$y1$binindex,rep=res1a$y1$rep), FUN='quantile', probs=c(0.9))

# calculate upper and lower limit of prediction interval
upperBoundaryp10 <- aggregate(percentile10$x, by=list(bin=percentile10$bin), FUN='quantile', probs=c(0.9))
lowerBoundaryp10 <- aggregate(percentile10$x, by=list(bin=percentile10$bin), FUN='quantile', probs=c(0.1))
upperBoundaryp50 <- aggregate(percentile50$x, by=list(bin=percentile50$bin), FUN='quantile', probs=c(0.9))
lowerBoundaryp50 <- aggregate(percentile50$x, by=list(bin=percentile50$bin), FUN='quantile', probs=c(0.1))
upperBoundaryp90 <- aggregate(percentile90$x, by=list(bin=percentile90$bin), FUN='quantile', probs=c(0.9))
lowerBoundaryp90 <- aggregate(percentile90$x, by=list(bin=percentile90$bin), FUN='quantile', probs=c(0.1))

# put ribbon limits in a data frame
df <- data.frame(
  bins = binsmiddle,
  uB10 = c(upperBoundaryp10$x[1],upperBoundaryp10$x,upperBoundaryp10$x[length(upperBoundaryp10$x)]),
  lB10 = c(lowerBoundaryp10$x[1],lowerBoundaryp10$x,lowerBoundaryp10$x[length(lowerBoundaryp10$x)]),
  uB50 = c(upperBoundaryp50$x[1],upperBoundaryp50$x,upperBoundaryp50$x[length(upperBoundaryp50$x)]),
  lB50 = c(lowerBoundaryp50$x[1],lowerBoundaryp50$x,lowerBoundaryp50$x[length(lowerBoundaryp50$x)]),
  uB90 = c(upperBoundaryp90$x[1],upperBoundaryp90$x,upperBoundaryp90$x[length(upperBoundaryp90$x)]),
  lB90 = c(lowerBoundaryp90$x[1],lowerBoundaryp90$x,lowerBoundaryp90$x[length(lowerBoundaryp90$x)]))

# plot de VPC
vpc1 <- ggplot()+
  geom_ribbon(data=df, aes(x=bins,ymin=lB10, ymax=uB10),fill='#b2d8ff')+
  geom_ribbon(data=df, aes(x=bins,ymin=lB50, ymax=uB50), fill="#fdcbcd")+
  geom_ribbon(data=df, aes(x=bins,ymin=lB90, ymax=uB90),fill='#b2d8ff')+
  geom_point(data=empirical, aes(x=bins_middles,y=empirical_lower), color="royalblue4")+
  geom_line(data=empirical, aes(x=bins_middles,y=empirical_lower), color="royalblue4")+
  geom_point(data=empirical, aes(x=bins_middles,y=empirical_upper), color="royalblue4")+
  geom_line(data=empirical, aes(x=bins_middles,y=empirical_upper), color="royalblue4")+
  geom_point(data=empirical, aes(x=bins_middles,y=empirical_median), color="royalblue4")+
  geom_line(data=empirical, aes(x=bins_middles,y=empirical_median), color="royalblue4")+
  theme_bw()+ylab("Predictions")+xlab("Time (days)")+ theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank(),
                                                            panel.background = element_blank(), axis.line = element_line(colour = "black"))


  
print(vpc1)




#===================================================================
# VPC with removal of observation points happening after dropout
#===================================================================

res1b <- res1a

# removing the concentrations measured after the dropout
dropouttable <- res1b$survival[seq(2,length(res1b$survival$time),by=2),1:3]
colnames(dropouttable)[3] <- "dropouttime"
res1b$y1 <- merge(x=res1b$y1,y=dropouttable,by=c("id","rep"))
res1b$y1 <- res1b$y1[res1b$y1$time < res1b$y1$dropouttime, ] 


# assign simulated data to bins
res1b$y1$binindex <- findInterval(res1b$y1$time, bins, all.inside = T)

# calculate percentiles for each replicate (data set) and each bin
percentile10 <- aggregate(res1b$y1$y1, by=list(bin=res1b$y1$binindex,rep=res1b$y1$rep), FUN='quantile', probs=c(0.1))
percentile50 <- aggregate(res1b$y1$y1, by=list(bin=res1b$y1$binindex,rep=res1b$y1$rep), FUN='quantile', probs=c(0.5))
percentile90 <- aggregate(res1b$y1$y1, by=list(bin=res1b$y1$binindex,rep=res1b$y1$rep), FUN='quantile', probs=c(0.9))

# calculate upper and lower limit of prediction interval
upperBoundaryp10 <- aggregate(percentile10$x, by=list(bin=percentile10$bin), FUN='quantile', probs=c(0.9))
lowerBoundaryp10 <- aggregate(percentile10$x, by=list(bin=percentile10$bin), FUN='quantile', probs=c(0.1))
upperBoundaryp50 <- aggregate(percentile50$x, by=list(bin=percentile50$bin), FUN='quantile', probs=c(0.9))
lowerBoundaryp50 <- aggregate(percentile50$x, by=list(bin=percentile50$bin), FUN='quantile', probs=c(0.1))
upperBoundaryp90 <- aggregate(percentile90$x, by=list(bin=percentile90$bin), FUN='quantile', probs=c(0.9))
lowerBoundaryp90 <- aggregate(percentile90$x, by=list(bin=percentile90$bin), FUN='quantile', probs=c(0.1))

# put ribbon limits in a data frame
df <- data.frame(
  bins = binsmiddle,
  uB10 = c(upperBoundaryp10$x[1],upperBoundaryp10$x,upperBoundaryp10$x[length(upperBoundaryp10$x)]),
  lB10 = c(lowerBoundaryp10$x[1],lowerBoundaryp10$x,lowerBoundaryp10$x[length(lowerBoundaryp10$x)]),
  uB50 = c(upperBoundaryp50$x[1],upperBoundaryp50$x,upperBoundaryp50$x[length(upperBoundaryp50$x)]),
  lB50 = c(lowerBoundaryp50$x[1],lowerBoundaryp50$x,lowerBoundaryp50$x[length(lowerBoundaryp50$x)]),
  uB90 = c(upperBoundaryp90$x[1],upperBoundaryp90$x,upperBoundaryp90$x[length(upperBoundaryp90$x)]),
  lB90 = c(lowerBoundaryp90$x[1],lowerBoundaryp90$x,lowerBoundaryp90$x[length(lowerBoundaryp90$x)]))

# plot de VPC
vpc2 <- ggplot()+
  geom_ribbon(data=df, aes(x=bins,ymin=lB10, ymax=uB10),fill='#b2d8ff')+
  geom_ribbon(data=df, aes(x=bins,ymin=lB50, ymax=uB50), fill="#fdcbcd")+
  geom_ribbon(data=df, aes(x=bins,ymin=lB90, ymax=uB90),fill='#b2d8ff')+
  geom_point(data=empirical, aes(x=bins_middles,y=empirical_lower), color="royalblue4")+
  geom_line(data=empirical, aes(x=bins_middles,y=empirical_lower), color="royalblue4")+
  geom_point(data=empirical, aes(x=bins_middles,y=empirical_upper), color="royalblue4")+
  geom_line(data=empirical, aes(x=bins_middles,y=empirical_upper), color="royalblue4")+
  geom_point(data=empirical, aes(x=bins_middles,y=empirical_median), color="royalblue4")+
  geom_line(data=empirical, aes(x=bins_middles,y=empirical_median), color="royalblue4")+
  theme_bw()+ylab("Predictions")+xlab("Time (days)")+ theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank(),
                                                            panel.background = element_blank(), axis.line = element_line(colour = "black"))


print(vpc2)



#===================================================================================================
# VPC with removal of PD observations after dropout AND assumption about new PD observation times
# when dropout happens later than in the original data set
#===================================================================================================


# read data set used in Monolix project
tt <- readDatamlx(project=paste0(project.name,".mlxtran"))


# output definition with regular observation times for all ids, as in dataset
outy1 <- list(name="y1", time=seq(-100,1500,by=100))


# simulate data sets (using replicates)
res1c <- simulx(project=paste0(project.name,".mlxtran"),
              output = outy1,
              nrep = 500)


# removing the concentrations measured after the dropout
dropouttable <- res1c$survival[seq(2,length(res1c$survival$time),by=2),1:3]
colnames(dropouttable)[3] <- "dropouttime"
res1c$y1 <- merge(x=res1c$y1,y=dropouttable,by=c("id","rep"))
res1c$y1 <- res1c$y1[res1c$y1$time < res1c$y1$dropouttime, ] 


# assign simulated data to bins
res1c$y1$binindex <- findInterval(res1c$y1$time, bins, all.inside = T)

# calculate percentiles for each replicate (data set) and each bin
percentile10 <- aggregate(res1c$y1$y1, by=list(bin=res1c$y1$binindex,rep=res1c$y1$rep), FUN='quantile', probs=c(0.1))
percentile50 <- aggregate(res1c$y1$y1, by=list(bin=res1c$y1$binindex,rep=res1c$y1$rep), FUN='quantile', probs=c(0.5))
percentile90 <- aggregate(res1c$y1$y1, by=list(bin=res1c$y1$binindex,rep=res1c$y1$rep), FUN='quantile', probs=c(0.9))

# calculate upper and lower limit of prediction interval
upperBoundaryp10 <- aggregate(percentile10$x, by=list(bin=percentile10$bin), FUN='quantile', probs=c(0.9))
lowerBoundaryp10 <- aggregate(percentile10$x, by=list(bin=percentile10$bin), FUN='quantile', probs=c(0.1))
upperBoundaryp50 <- aggregate(percentile50$x, by=list(bin=percentile50$bin), FUN='quantile', probs=c(0.9))
lowerBoundaryp50 <- aggregate(percentile50$x, by=list(bin=percentile50$bin), FUN='quantile', probs=c(0.1))
upperBoundaryp90 <- aggregate(percentile90$x, by=list(bin=percentile90$bin), FUN='quantile', probs=c(0.9))
lowerBoundaryp90 <- aggregate(percentile90$x, by=list(bin=percentile90$bin), FUN='quantile', probs=c(0.1))

# put ribbon limits in a data frame
df <- data.frame(
  bins = binsmiddle,
  uB10 = c(upperBoundaryp10$x[1],upperBoundaryp10$x,upperBoundaryp10$x[length(upperBoundaryp10$x)]),
  lB10 = c(lowerBoundaryp10$x[1],lowerBoundaryp10$x,lowerBoundaryp10$x[length(lowerBoundaryp10$x)]),
  uB50 = c(upperBoundaryp50$x[1],upperBoundaryp50$x,upperBoundaryp50$x[length(upperBoundaryp50$x)]),
  lB50 = c(lowerBoundaryp50$x[1],lowerBoundaryp50$x,lowerBoundaryp50$x[length(lowerBoundaryp50$x)]),
  uB90 = c(upperBoundaryp90$x[1],upperBoundaryp90$x,upperBoundaryp90$x[length(upperBoundaryp90$x)]),
  lB90 = c(lowerBoundaryp90$x[1],lowerBoundaryp90$x,lowerBoundaryp90$x[length(lowerBoundaryp90$x)]))

# plot de VPC
vpc3 <- ggplot()+
  geom_ribbon(data=df, aes(x=bins,ymin=lB10, ymax=uB10),fill='#b2d8ff')+
  geom_ribbon(data=df, aes(x=bins,ymin=lB50, ymax=uB50), fill="#fdcbcd")+
  geom_ribbon(data=df, aes(x=bins,ymin=lB90, ymax=uB90),fill='#b2d8ff')+
  geom_point(data=empirical, aes(x=bins_middles,y=empirical_lower), color="royalblue4")+
  geom_line(data=empirical, aes(x=bins_middles,y=empirical_lower), color="royalblue4")+
  geom_point(data=empirical, aes(x=bins_middles,y=empirical_upper), color="royalblue4")+
  geom_line(data=empirical, aes(x=bins_middles,y=empirical_upper), color="royalblue4")+
  geom_point(data=empirical, aes(x=bins_middles,y=empirical_median), color="royalblue4")+
  geom_line(data=empirical, aes(x=bins_middles,y=empirical_median), color="royalblue4")+
  theme_bw()+ylab("Predictions")+xlab("Time (days)")+ theme(panel.grid.major = element_blank(), panel.grid.minor = element_blank(),
                                                            panel.background = element_blank(), axis.line = element_line(colour = "black"))


print(vpc3)


